package ui.controller;

import java.awt.Point;
import java.util.ArrayList;

import Interfaces.CategoryListener;
import Interfaces.ObjectListener;
import classes.CpsEdge;
import classes.CpsObject;
import classes.HolonObject;
import classes.HolonSwitch;
import classes.Position;
import ui.model.Model;

public class CanvasController {

	private Model MODEL;
	private MultiPurposeController mpC;

	public CanvasController(Model model, MultiPurposeController mp) {
		this.MODEL = model;
		this.mpC = mp;
	}

	/**
	 * Add an CpsObject to the model and notify the ObjectListener for update.
	 * 
	 * @param object
	 *            CpsObject to be added.
	 */
	public void addObject(CpsObject object) {
		MODEL.getCvsObjIdx().put(object.getID(), MODEL.getObjectsOnCanvas().size());
		MODEL.getObjectsOnCanvas().add(object);
		notifyObjListeners();
	}

	/**
	 * 
	 * @param object
	 */
	public void addNewObject(CpsObject object) {
		object.setSav("CVS");
		object.setConnections(new ArrayList<CpsEdge>());
		addObject(object);
	}

	public void addObjectListener(ObjectListener objLis) {
		MODEL.getObjectListeners().add(objLis);
	}

	/**
	 * notifies all listeners about changes in the Canvas
	 */
	public void notifyObjListeners() {
		for (ObjectListener l : MODEL.getObjectListeners()) {
			l.onChange(MODEL.getObjectsOnCanvas());
		}
	}

	/**
	 * Deletes an CpsObject on the Canvas and its connections
	 * 
	 * @param CpsObject
	 */
	public void deleteObjectOnCanvas(CpsObject obj) {
		CpsEdge e = null;
		for (CpsObject cps : MODEL.getObjectsOnCanvas()) {
			for (CpsEdge p : cps.getConnections()) {
				if (p.getA() == obj || p.getB() == obj) {
					e = p;
				}
			}
			cps.getConnectedTo().remove(e);
			MODEL.getEdgesOnCanvas().remove(e);
		}
		mpC.decIdx(obj.getID(), MODEL.getCvsObjIdx());
		MODEL.getCvsObjIdx().remove(obj.getID());
		MODEL.getObjectsOnCanvas().remove(obj);

	}

	/**
	 * @param objectsOnCanvas
	 *            the objectsOnCanvas to set
	 */
	public void addEdgeOnCanvas(CpsEdge edge) {
		MODEL.getEdgesOnCanvas().add(edge);
	}

	/**
	 * @param edgesOnCanvas
	 *            the edge to remove
	 */
	public void removeEdgesOnCanvas(CpsEdge edge) {
		edge.getA().getConnections().remove(edge);
		edge.getB().getConnections().remove(edge);
		MODEL.getEdgesOnCanvas().remove(edge);
	}

	/**
	 * Copy all Selected Objects
	 */
	public void copyObjects() {
		MODEL.setClipboradObjects((ArrayList<CpsObject>) MODEL.getSelectedCpsObjects().clone());
	}

	/**
	 * Paste all Selected Objects
	 * 
	 * @param Point
	 *            the mouse Position
	 */
	public void pasteObjects(Point p) {
		System.out.println("paste");
		MODEL.getSelectedCpsObjects().clear();
		CpsObject tCps = null;
		int x = Integer.MAX_VALUE, y = Integer.MAX_VALUE;
		for (CpsObject cps : MODEL.getClipboradObjects()) {
			if (cps.getPosition().x < x) {
				x = cps.getPosition().x;
			}
			if (cps.getPosition().y < y) {
				y = cps.getPosition().y;
			}
		}

		for (CpsObject cps : MODEL.getClipboradObjects()) {
			if (cps instanceof HolonObject) {
				tCps = new HolonObject((HolonObject) cps);
			} else if (cps instanceof HolonSwitch) {
				tCps = new HolonSwitch((HolonSwitch) cps);
			}
			tCps.setPosition(new Position(p.x + (cps.getPosition().x-x), p.y + (cps.getPosition().y-y)));
			addObject(tCps);
			MODEL.getSelectedCpsObjects().add(tCps);
		}
	}

	/**
	 * Cut all Selected Objects
	 */
	public void cutObjects() {
		MODEL.setClipboradObjects((ArrayList<CpsObject>) MODEL.getSelectedCpsObjects().clone());
		for (CpsObject cps : MODEL.getSelectedCpsObjects()) {
			deleteObjectOnCanvas(cps);
		}
		MODEL.getSelectedCpsObjects().clear();
	}
}
